@@ -5,9 +5,7 @@
 
 use fri::FriOptions;
 use math::StarkField;
-use utils::{
-    string::ToString, ByteReader, ByteWriter, Deserializable, DeserializationError, Serializable,
-};
+use utils::{ByteReader, ByteWriter, Deserializable, DeserializationError, Serializable};
 
 // TYPES AND INTERFACES
 // ================================================================================================
@@ -56,6 +54,8 @@ pub enum FieldExtension {
     None = 1,
     /// Composition polynomial is constructed in the quadratic extension of the base field.
     Quadratic = 2,
+    /// Composition polynomial is constructed in the cubic extension of the base field.
+    Cubic = 3,
 }
 
 /// STARK protocol parameters.
@@ -255,6 +255,7 @@ impl FieldExtension {
         match self {
             Self::None => 1,
             Self::Quadratic => 2,
+            Self::Cubic => 3,
         }
     }
 }
@@ -272,9 +273,10 @@ impl Deserializable for FieldExtension {
         match source.read_u8()? {
             1 => Ok(FieldExtension::None),
             2 => Ok(FieldExtension::Quadratic),
+            3 => Ok(FieldExtension::Cubic),
             value => Err(DeserializationError::InvalidValue(format!(
                 "value {} cannot be deserialized as FieldExtension enum",
-                value.to_string()
+                value
             ))),
         }
     }
@@ -310,7 +312,7 @@ impl Deserializable for HashFunction {
             3 => Ok(HashFunction::Sha3_256),
             value => Err(DeserializationError::InvalidValue(format!(
                 "value {} cannot be deserialized as HashFunction enum",
-                value.to_string()
+                value
             ))),
         }
     }
