@@ -20,7 +20,7 @@ use utils::{
 ///
 /// Internally, the commitments are stored as a sequence of bytes. Thus, to retrieve the
 /// commitments, [parse()](Commitments::parse) function should be used.
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Default, Eq, PartialEq)]
 pub struct Commitments(Vec<u8>);
 
 impl Commitments {
@@ -77,12 +77,6 @@ impl Commitments {
     }
 }
 
-impl Default for Commitments {
-    fn default() -> Self {
-        Commitments(Vec::new())
-    }
-}
-
 impl Serializable for Commitments {
     /// Serializes `self` and writes the resulting bytes into the `target`.
     fn write_into<W: ByteWriter>(&self, target: &mut W) {
